;;; schemed-html.scm

(define *linktable-file* "/home/cmwilson/html/meta/linktable")
(define *link-table* (with-input-from-file *linktable-file* read))
(define *heading* 0)  ; counter for <h?>

(define schemed-html->html
  (lambda (schemed-html-text)
    (let ((schemed-html-port (make-string-input-port schemed-html-text)))
      (if (eq? 'schemed-html-5 (read schemed-html-port))
	  (eval (read schemed-html-port) (interaction-environment))
	  (error "schemed-html->html: file is not recognized as schemed-html")))))

(define doctype-html4.0 (list "<!doctype html public \"-//W3C//DTD HTML 4.0//EN\" "
			      "\"http://www.w3.org/TR/REC-html40/strict.dtd\">"))

(define link-lookup
  (lambda (name)
    (cond ((assq name *link-table*) => cadr)
	  (else (format (error-output-port) "link-lookup: unknown link identifier: ~a~%" name)
		(list "Unknown-link-identifier: " (symbol->string name))))))

(define link
  (lambda (text link . maybe-info)
    (list "<a href=\""
	  (cond ((string? link)
		 ;(warning "link is a string")
                 link)
		((symbol? link) (link-lookup link))
		(else (warning "link is neither a string nor link")))
	  "\">" text "</a>" maybe-info)))

(define with-title
  (lambda (title . body)
    (list doctype-html4.0 "\n\n" "<html>\n" "<head><title>" title "</title></head>\n"
	  "<body>\n" body "\n</body>\n" "</html>\n")))

(define with-heading
  (lambda (heading-text . body)
    (dynamic-wind
	(lambda ()
	  (set! *heading* (+ *heading* 1)))
	(lambda ()
	  (let ((heading (if (<= 0 *heading* 6)
			     *heading*
			     6)))
	    (list "\n<h" heading ">" heading-text "</h" heading ">\n" body)))
	(lambda ()
	  (set! *heading* (- *heading* 1))))))

(define listing
  (lambda links
    (list "<ul>\n"
	  (map (lambda (link)
		 (list "<li>" link "\n"))
	       links)
	  "</ul>\n")))

(define separate-with
  (lambda (separator)
    (lambda links
      (let loop ((links links))
	(cond ((null? links) '())
	      ((null? (cdr links)) links)
	      (else (cons (car links)
			  (cons separator (loop (cdr links))))))))))

(define dot-sep (separate-with " . "))
(define and-sep (separate-with " and "))


;;; (indented-listing (category <name> <link>*)*)

(define category
  (lambda (name . links)
    (list "<tr><td>" name ":\n<td>"
	  (apply (separate-with " .\n") links))))
 
(define indented-listing
  (lambda categories
    (list "<table cellpadding=\"0\" cellspacing=\"0\">\n"
	  (apply (separate-with "\n") categories)
	  "\n</table>\n")))

(define comment
  (lambda strs
    (list "\n<!-- " strs " -->")))

(define p (lambda () "\n<p>\n\n"))
(define br (lambda () "<br>\n"))

(define cite
  (lambda quotes
    (list "<cite>" quotes "</cite>")))

(define usage
  (lambda (prog-name)
    (format #t (string-append "usage:  ~a <filename> <new-filename>~%"
			      "  <filename> is the scheme-html source and~%"
			      "  <new-filename> is the file in which to write the html output.~%"
			      "  If <new-filename> is not specified, it defaults to the standard output.~%")
	    prog-name)))

(define main
  (lambda (args)
    (let* ((run-contents
	    (lambda ()
	      (if (eq? 'schemed-html-5 (read))
		  (eval (read) (interaction-environment))
		  (error (format #f (string-append "main: unrecognized header symbol~%"
						   "We were looking for the first symbol"
						   " of the file to be schemed-html-5~%")))))))
      (case (length args)
	((2) (let ((input-file (cadr args)))
	       (with-input-from-file input-file run-contents)))
	((3) (let ((input-file (cadr args))
		   (output-file (caddr args)))
	       (with-output-to-file output-file
		 (lambda ()
		   (with-input-from-file input-file run-contents)))))
	(else (let ((progname (car args)))
		(usage progname)))))))
