;;; symlink-tree.scm

;;; make a symlink tree
;;; dest is the directory where things will end up.
;;; source can be any existing file.
;;; if source is a file, symlink the file.
;;; if source is a directory, create an actual directory and recurse on the source's contents.

(define symlink-tree
  (lambda (source destination)
    (cond ((not (file-directory? destination))
	   (error "destination must be a directory"))
	  ((file-not-exists? source)
	   (error "source doesn't exist: ~a" source))
	  (else
	   ;; want least significant file or directory name
	   (let ((name (last (split-file-name source))))
	     (with-cwd destination
	       (if (file-directory? source)
		   ;; directory
		   (begin
		     (if (and (file-exists? name)
			      (file-directory? name))
			 (create-directory name)
			 (warning "file exists but should be directory" name))
		     (let ((source (expand-file-name source (cwd))))  ; make source absolute
		       (with-cwd name
			 (for-each (lambda (file)
				     (symlink-tree (string-append source "/" file) "."))
				   (directory-files source)))))
		   ;; file
		   (if (and (file-exists? name)
			    (file-symlink? name))
		       (create-symlink source name #t)
		       (warning "file exists but isn't symlink: ~a" name)))))))))
