;;; helper-functions.scm

(define-syntax when
  (syntax-rules ()
    ((when condition body ...)
     (if condition
	 (begin body ...)
	 #f))))

(define-syntax unless
  (syntax-rules ()
    ((unless condition body ...)
     (if condition
	 #t
	 (begin body ...)))))


;;; Normal case uses eqv? (well, memv).

(define-syntax case-equal
  (syntax-rules (else)
    ((case-equal (key ...) clauses ...)
     (let ((atom-key (key ...)))
       (case-equal atom-key clauses ...)))
    ((case-equal key (else result1 result2 ...))
     (begin result1 result2 ...))
    ((case-equal key ((atoms ...) result1 result2 ...))
     (if (member key '(atoms ...))
	 (begin result1 result2 ...)))
    ((case-equal key ((atoms ...) result1 result2 ...) clause clauses ...)
     (if (member key '(atoms ...))
	 (begin result1 result2 ...)
	 (case-equal key clause clauses ...)))))


;;; bind is a mess because of "Warning: segment matching not
;;; implemented" in scheme48

(define-syntax bind
  (syntax-rules ()
    ((bind () body ...)
     (begin body ...))
    ((bind ((var1 def)) body ...)
     (let ((var def)) body ...))
    ((bind ((var1 var2 def)) body ...)
     (call-with-values
       (lambda () def)
       (lambda (var1 var2) body ...)))
    ((bind ((var1 var2 var3 def)) body ...)
     (call-with-values
       (lambda () def)
       (lambda (var1 var2 var3) body ...)))
    ((bind ((var1 var2 var3 var4 def)) body ...)
     (call-with-values
       (lambda () def)
       (lambda (var1 var2 var3 var4) body ...)))
    ((bind ((var1 var2 var3 var4 var5 def)) body ...)
     (call-with-values
       (lambda () def)
       (lambda (var1 var2 var3 var4 var5) body ...)))
    ((bind ((var1 var2 var3 var4 var5 var6 def)) body ...)
     (call-with-values
       (lambda () def)
       (lambda (var1 var2 var3 var4 var5 var6) body ...)))
    ((bind ((var1 var2 var3 var4 var5 var6 var7 def)) body ...)
     (call-with-values
       (lambda () def)
       (lambda (var1 var2 var3 var4 var5 var6 var7) body ...)))
    ((bind ((var1 var2 var3 var4 var5 var6 var7 var8 def)) body ...)
     (call-with-values
       (lambda () def)
       (lambda (var1 var2 var3 var4 var5 var6 var7 var8) body ...)))
    ((bind (vars-def morevars-def ...) body ...)
     (bind (vars-def)
       (bind (morevars-def ...) body ...)))
    ))


;;; Makes a string out of everything sitting at port.

(define read-all
  (lambda (port)
    (apply string-append
	   (let loop ()
	     (let ((data (or (read-string/partial 1000 port) "")))
	       (if (string=? "" data)
		   '()
		   (cons data (loop))))))))


;;; Make the file executable if it isn't already.

(define ensure-executable
  (lambda (filename)
    (set-file-mode filename
		   (bitwise-ior #b001001001 (file-mode filename #t)))))

;;; tree-for-each

(define tree-for-each
  (lambda (f tree)
    (cond ((null? tree) #t)
	  ((pair? tree)
	   (tree-for-each f (car tree))
	   (tree-for-each f (cdr tree)))
	  (else (f tree)))))

;;; (warning message)

(define-syntax warning
  (syntax-rules ()
    ((warning message ...)
     (for-each (lambda (m)
		 (format (error-output-port) "WARNING:  ~a~%" m))
	       (list message ...)))))


(define last
  (lambda (list)
    (let loop ((list list) (last-item list))
      (if (null? list)
	  last-item
	  (loop (cdr list) (car list))))))


;;; find the most recently altered file in source.
;;; source may be a directory or a file.

(define latest
  (lambda (source)
    (cond ((file-not-exists? source)
	   (error "source must exist: ~a" source))
	  ((file-directory? source)
	   (with-cwd source
             (apply max 0 (map latest (directory-files)))))
	  (else (file-last-mod source)))))
